
# q-expansion of newform 28.10.a.b, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 28
weight = 10
poly_data = [-2802, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [-35, -1], [777, -19], [-2401, 0], [31194, -182], [61383, 87], [36792, 2986], [585599, -2019], [1131192, -7448], [-961680, 24878], [1488942, 46242], [-6709264, -136122], [-18183900, -14922], [-10982458, -52626], [-681366, 167910], [-8949306, 702240], [112355271, -312307], [-42923559, 1314933], [89784436, 1498728], [115689084, -30548], [44049166, -15252], [-92137092, 2849700], [312320547, 3896617], [-787388574, -2093256], [106832992, 2735970], [-602920983, 6181517], [-208194238, -7117434], [75962448, 18350388], [112975180, 4817106], [-61904508, 2397206], [-2119813548, -17933580], [-25935609, 12014837], [3329621862, 45405668], [3006933433, 4998219], [-3097344006, -70748580], [1383735652, -49533876], [710554943, 64153935], [-15846239338, 3200694], [14595301350, 34685234], [12392771979, -83711725], [14484581496, -47103084], [13994648535, 66925935], [6606838500, 58488444], [8875460624, -21505974], [19454701914, 201806332], [-3510662722, -218491350], [1824284108, -505287552], [-1510268452, 47657076], [-8665953009, 44113149], [-29582774501, 90264795], [-17813033406, 272655880], [16751260248, -15382416], [-27254071432, -452227782], [-25926377799, -5870613], [49893929274, -674319000], [70954505148, -298110708], [39759592635, 598545539], [-227451000, 1307726040], [71108044550, -503454672], [-39968949942, 575719984], [-32710767257, 347558973], [-90738271155, -828352215], [-191263304057, -27155163], [-146831051808, -1202851000], [-19349201900, 1361237814], [184871056338, -308365092], [-31519522186, 1532071422], [99208232040, -2408481054], [-53441405394, 666789942], [189991231003, 928897491], [-184030828254, 210060816], [-173991441804, 3811819004], [68854516052, -2325786132], [589061721562, 775901532], [195451635730, 4264245426], [33194026698, -2925096266], [268293185304, -947744770], [467272264347, 2204104995], [-656690226816, -1423839942], [403107760160, -862898502], [406412788929, -5999499909], [-1113895389426, -397271784], [61195932192, 3974970552], [385349341580, -4794758634], [-424612939128, 6908967888], [-850709883468, -1825474448], [-937619592774, -5434528120], [-125989025816, 1720427226], [-76900554801, -3575360497], [790706957920, -7272096048], [568571555979, 3112639313], [584366770974, -2903625134], [2151959598776, 494426520], [-510372945156, 9106740888], [159747963796, -7536223800], [1284176227908, 13715532852], [559256660571, -6691495725], [425953717308, -11586023922], [-167479810813, -17778776319], [-740574278598, 9570317316], [-671902512798, 26424363882], [-61404375990, 12424941620], [-1535300825331, -5157701201], [1973510890704, 16149782418], [-2385104628966, 20838733482], [3717681871498, 8915170440], [2996866595373, -8652086537], [3896220897198, 5756859780], [1570378697472, 13830474504], [-1938401070046, -14199737832], [1349997992784, 10821276432], [-868770091128, 26533834110], [4082378276304, 6158091730], [-3024929143481, 806973933], [-1031021169964, -21964681452], [-5065538331192, -9809973278], [-4853956373203, 15665252031], [-802612219122, -18173317638], [327334017228, -19984739838], [4144629873690, 32491370338], [-2885847093227, 23807861745], [1718595304790, 8024212812], [2394507017007, -20374339737], [3001258001364, -67552646448], [-1242194897041, -28265399523], [-4116844200060, -25827933670], [4407039810248, 10665575214], [5004956391414, -39306076614], [6669735424618, -9808418082], [-3258377885679, 63619818621], [3350442911670, 48790100310], [2202561674748, 51427840884], [-1613803318128, 104338931592], [-3597566149392, -51050607930], [969488655912, -51935125310], [4298943937204, 84276050094], [-4071109303779, -83714830663], [2622379674579, -95227092279], [-1285362959853, -64472264325], [-13224240304176, -77527280110], [1745261736715, 98942188617], [-19679871725922, 54102499864], [7129567034848, -70160648040], [-1002397020864, 3711710772], [5063333222353, 27893749437], [5834817080778, -37969993538], [7486345023227, 201103444299], [-979923498960, 1010607114], [-18396528507175, 33208133643], [-7253823002100, -134354152492], [-9798816693272, 132256405974], [15846747482622, 122983922340], [762532470800, -211580831028], [9016050354738, -206135198250], [-9440916216792, 106718811948], [4740825925380, 208176641316], [-9070097460608, -18119682504], [-1071835532880, -16130287326], [-5050427316802, 211435358460], [1673808470481, -266872895827], [-13380563624574, 210781416986], [-3360799249182, -121644439112], [-36061118373520, 79049894784], [27696824004927, -96045909723], [-38084639341746, -125514613308], [-6215913253158, -192589548450], [-2413249905688, 72207039912], [-13880648585083, 3964177137]]
